/*
 * Copyright (c) 2008 Nick Kossifidis <mickflemm@gmail.com>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

/* Parameter values as defined on patent doc */

/* Table 1 params - SSCI immunity levels (values in half dB) */
#if defined(_ATH5K_ANI)
const	s8 coarse_high[]	= { -18, -18, -16, -14, -12 };
const	s8 coarse_low[]		= { -52, -56, -60, -64, -70 };
const	s8 total_size[]		= { -34, -41, -48, -55, -62 };
const	s8 fir_pwr[]		= { -70, -72, -75, -78, -80 };
#endif

#define	AR5K_ANI_MIN_SSCI_IMMUNITY_LEVEL	0
#define AR5K_ANI_MAX_SSCI_IMMUNITY_LEVEL	4

/* Table 2 - OFDM weak signal detection thresholds */
#define	AR5K_ANI_OFDM_WEAK_SIG_M1_OFF			127
#define	AR5K_ANI_OFDM_WEAK_SIG_M2_OFF			127
#define	AR5K_ANI_OFDM_WEAK_SIG_M2_COUNT_HIGH_OFF	31
#define	AR5K_ANI_OFDM_WEAK_SIG_M2_COUNT_LOW_OFF		63

#define	AR5K_ANI_OFDM_WEAK_SIG_M1_HIGH_ON		77
#define	AR5K_ANI_OFDM_WEAK_SIG_M2_HIGN_ON		64
#define	AR5K_ANI_OFDM_WEAK_SIG_M1_LOW_ON		50
#define	AR5K_ANI_OFDM_WEAK_SIG_M2_LOW_ON		40
#define	AR5K_ANI_OFDM_WEAK_SIG_M2_COUNT_HIGH_ON		16
#define	AR5K_ANI_OFDM_WEAK_SIG_M2_COUNT_LOW_ON		48

/* Current ani parameters */
struct ath5k_ani_state {

	/* Used to track down progress */
	DECLARE_BITMAP(state, 8);

	/* Index to currently used
	 * ssci_immunity_params */
	u8	ssci_immunity_level;

	/* Weak signal detection
	 * params (dB - hw uses half dB so
	 * they need to be doubled) */
	u8	ofdm_cyc_rssi_thr;
	u8	cck_weak_sig_thr;
	u8	firstep;

	u8	success_count;

};

/* state bits */
#define	ATH5K_ANI_RUNNING			0
#define	ATH5K_ANI_POLL_RUNNING			1
#define	ATH5K_ANI_OFDM_WEAK_SIG_DET_ACTIVE	2
#define ATH5K_ANI_FALSE_DETECT_ABOVE_HIGH	3
#define ATH5K_ANI_FALSE_DETECT_ABOVE_LOW	4
#define	ATH5K_ANI_SSCI_POLL_DONE		5
#define	ATH5K_ANI_OFDM_WEAK_SIG_POLL_DONE	6
#define	ATH5K_ANI_CCK_WEAK_SIG_POLL_DONE	7

#define ATH5K_SSCI_IMMUNITY_LEVEL_MIN	0
#define ATH5K_SSCI_IMMUNITY_LEVEL_MAX	4
#define ATH5K_OFDM_CYC_RSSI_THR_MIN	1	/* 2 */
#define ATH5K_OFDM_CYC_RSSI_THR_MAX	8	/* 16 */
/* #define ATH5K_CCK_WEAK_SIG_THR_MIN	1 */	/* 2?*/
#define	ATH5K_CCK_WEAK_SIG_THR_MIN	3	/* 6 */
#define ATH5K_CCK_WEAK_SIG_THR_MAX	4	/* 8 */
#define	ATH5K_FIRSTEP_MIN		0	/* Max cck sensitivity */
#define	ATH5K_FIRSTEP_MAX		4	/* 8 ? */
#define ATH5K_ANI_MAX_SUCCESS_COUNT	10	/* ? */

/* Stats used for evaluating ANI parameters */
struct ath5k_ani_stats {
	/* False detect thresholds */
	u16	ofdm_false_detect_high;
	u16	ofdm_false_detect_low;
	u16	cck_false_detect_high;
	u16	cck_false_detect_low;

	/* Current false detect counts */
	u16	ofdm_false_detects;
	u16	cck_false_detects;
	/* False detect counts of previous
	 * measurement used for comparison */
	u16	prev_ofdm_false_detects;
	u16	prev_cck_false_detects;

	/* How long it takes to do a
	 * "many false detects" check */
	u32	measurement_start;
	u32	max_measurement_duration;
};

/* Got these from ath9k */
#define ATH5K_ANI_OFDM_FALSE_DETECT_HIGH	500
#define ATH5K_ANI_OFDM_FALSE_DETECT_LOW		200
#define ATH5K_ANI_CCK_FALSE_DETECT_HIGH		200
#define ATH5K_ANI_CCK_FALSE_DETECT_LOW		100
#define	ATH5K_ANI_MAX_MEASUREMENT_DURATION	100
